#pragma once

#include <TApplication.h>
#include <TGClient.h>
#include <TCanvas.h>
#include "TMath.h"
#include "TH2F.h"
#include "TGraph.h"
#include "TStyle.h"
#include "TEllipse.h"
#include <TGButton.h>
#include <TRootEmbeddedCanvas.h>
#include <TGFrame.h>
#include <TGTab.h>
#include <TGMenu.h>
#include <TGNumberEntry.h>
#include <TGLabel.h>
#include <TGComboBox.h>
#include <TGFileDialog.h>

#include "setup.h"

class MyMainFrame : public TGMainFrame {
private:
    TGMainFrame         *fMain;
    // // tab
    // TGTab *fTab;
    //buttons
    TGCompositeFrame* fF3;
    TGLayoutHints *fL3;
    TGTextButton *fStartB;
    TGTextButton *fStopB;
    TGTextButton *fClearB;
    // canvas
    TGCompositeFrame *fF5;
    TGLayoutHints *fL4;
    TRootEmbeddedCanvas *fEcanvas;
    TCanvas* canvas;
    TH2D* histo;
    void initHist();
    // gridlines
    const int Nl = 19; // Number of drawn latitudes
    const int NL = 19; // Number of drawn longitudes
    int M  = 30;
    std::vector<TGraph*> latitudes;
    std::vector<TGraph*> longitudes;
    void drawGridlines();
    // source marker
    TEllipse* sourceMarker;
    void drawSource();
    // configurations
    const Setup* config;
    int Init();
    void redraw();
public:
    // running status
    bool stop=true;
    ULong64_t counts=0;
    MyMainFrame(const Setup* config_, const TGWindow *p,UInt_t w,UInt_t h);
    virtual ~MyMainFrame();
    void addCone(const std::vector<Cone> cones);
    // slots
    void Start();
    void Stop();
    void Clear();
    void CloseWindow();

    ClassDef(MyMainFrame,0)
};

int aitoff2xy(const double& l, const double& b, double &Al, double &Ab);