#pragma once

#include <vector>
#include <math.h>
#include <stdexcept>

class Vector3D
{
public:
    double X=0;
    double Y=0;
    double Z=0;
    Vector3D():Vector3D(0, 0, 0) {}
    Vector3D(const double x, const double y, const double z): X(x), Y(y), Z(z) {}

    Vector3D operator+(const Vector3D& r) const {
        return Vector3D(this->X + r.X, this->Y + r.Y, this->Z + r.Z);
    }
    Vector3D operator-(const Vector3D& r) const {
        return Vector3D(this->X - r.X, this->Y - r.Y, this->Z - r.Z);
    }

    double operator*(const Vector3D& r) const {
        return this->X * r.X + this->Y * r.Y + this->Z * r.Z;
    }

};


inline double getCosAngle(const Vector3D& l, const Vector3D& r) {
    return l*r / std::sqrt((l*l) * (r*r));
}

class Setup
{
private:
    /* data */
public:
    double pitchX = 11;
    int      numX =  7;
    double pitchY = 13;
    int      numY =  4;
    double pitchZ = 35;
    int      numZ =  2;
    // pixel coordinates, mm
    std::vector<Vector3D> pixels;
    int getPixelPos();
    // radius of projection sphere,  mm
    double R = 500;
    Setup(): Setup(500) {}
    Setup(const double r):R(r) {
        getPixelPos(); 
        pixelateSphere();
        randSource();
    }

    // image setup
    // Altitude angle, rad
    const int thetaBins = 180;
    const int phiBins = 360;
    const double dtheta = 180 / double(thetaBins) * M_PI / 180;
    const double thetaMin = -90 * M_PI / 180 + dtheta / 2;
    const double thetaMax = 90 * M_PI / 180 - dtheta / 2;
    // Azimuthal angle, rad
    const double dphi = 2 * M_PI /double(phiBins);
    const double phiMin = -M_PI + dphi / 2;
    const double phiMax = M_PI - dphi / 2;
    // pixelate the projection sphere
    std::vector<std::vector<Vector3D>> xbs;
    int pixelateSphere();

    // simulation setup
    const int nCones= 100;
    double trueTheta;
    double truePhi;
    Vector3D trueSource;
    int randSource();

    // refresh every 10 events
    const int chuckSize = 10;
};

class Cone
{
public:
    // interaction site 1 is the apex
    Vector3D apex;
    // site 2 - site 1 is the axis
    Vector3D axis;
    // half angle = abs (scattering angle)
    // for neutron, 0 < half angle < pi /2
    // for gamma,   0 < half angle < pi
    double cosHalfAngle;
    Cone(): Cone(Vector3D(0,0,0), Vector3D(0,0,0), 0) {}
    Cone(const Vector3D point, const Vector3D line, const double t): apex(point), axis(line), cosHalfAngle(t) {}
};


